unit uIntersect; { mk, 19.2.2004, Sedgewick, Algorithmen, S. 402/403 }

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls;

type
  TZustand = (z0,z1,z2,z3);         // selbst definierter Datentyp

  TPunkt = record                   // record ist wie class ,nur
             x,y : real             // ohne Methoden, ohne private
           end;

  T2Strecken = class(TObject)
  private
    p         : array[0..3] of TPunkt;
  public
    procedure setP(nr : integer; x,y : real);
    function getP(nr : integer) : TPunkt;
    function getIntersect : boolean;
  end;

  TForm1 = class(TForm)
    bZeichne: TButton;
    eP0x: TEdit; lP0x: TLabel; eP0y: TEdit; lP0y: TLabel;
    eP1x: TEdit; lP1x: TLabel; eP1y: TEdit; lP1y: TLabel;
    eP2x: TEdit; lP2x: TLabel; eP2y: TEdit; lP2y: TLabel;
    eP3x: TEdit; lP3x: TLabel; eP3y: TEdit; lP3y: TLabel;
    procedure FormMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure bZeichneClick(Sender: TObject);
  private
    zustand  : TZustand;
    Strecken : T2Strecken;
  public
    { Public-Deklarationen }
  end;

var
  Form1: TForm1;

implementation
{$R *.dfm}

procedure T2Strecken.setP(nr : integer; x,y : real);
begin
  p[nr].x := x;
  p[nr].y := y;
end;

function T2Strecken.getP(nr : integer) : TPunkt;
begin
  result.x := p[nr].x;
  result.y := p[nr].y;
end;

function T2Strecken.getIntersect : boolean;

  function ccw(p0,p1,p2 : TPunkt) : integer;
  var
    dx1,dx2,dy1,dy2 : real;
  begin
    dx1 := p1.x - p0.x; dy1 := p1.y - p0.y;
    dx2 := p2.x - p0.x; dy2 := p2.y - p0.y;
    if dx1*dy2 > dy1*dx2 then result := +1;
    if dx1*dy2 < dy1*dx2 then result := -1;
    if dx1*dy2 = dy1*dx2 then
    begin
      if (dx1*dx2 < 0) or (dy1*dy2 < 0) then
      result := -1
      else
        if (dx1*dx1 + dy1*dy1) >= (dx2*dx2 + dy2*dy2) then
          result := 0
        else
          result := 1;
    end;
  end;

  function intersect(p0,p1,p2,p3 : TPunkt) : boolean;
  begin
    result := (ccw(p0,p1,p2)*ccw(p0,p1,p3) <= 0) and
              (ccw(p2,p3,p0)*ccw(p2,p3,p1) <= 0)
  end;

begin
  result := intersect(p[0],p[1],p[2],p[3]);
end;

procedure TForm1.FormMouseDown(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  case zustand of                                         
    z0 : begin
           Strecken.setP(0,x,y);
           zustand := z1;
           canvas.Pixels[x,y] := clRed;
           canvas.MoveTo(x,y);
         end;
    z1 : begin
           Strecken.setP(1,x,y);
           zustand := z2;
           canvas.Pen.Color := clRed;
           canvas.LineTo(x,y);
         end;
    z2 : begin
           Strecken.setP(2,x,y);
           zustand := z3;
           canvas.Pixels[x,y] := clGreen;
           canvas.Pen.Color := clGreen;
           canvas.MoveTo(x,y);
         end;
    z3 : begin
           Strecken.setP(3,x,y);
           zustand := z0;
           canvas.LineTo(x,y);
           if Strecken.getIntersect then
             canvas.TextOut(x,y,'x')
           else
             canvas.TextOut(x,y,'0');
         end;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
  Strecken := T2Strecken.Create;
  zustand := z0;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  Strecken.Free;
end;

procedure TForm1.bZeichneClick(Sender: TObject);
var
  p0x,p0y,p1x,p1y,p2x,p2y,p3x,p3y : real;
begin
  p0x := StrToFloat(eP0x.Text); p0y := StrToFloat(eP0y.Text);
  p1x := StrToFloat(eP1x.Text); p1y := StrToFloat(eP1y.Text);
  p2x := StrToFloat(eP2x.Text); p2y := StrToFloat(eP2y.Text);
  p3x := StrToFloat(eP3x.Text); p3y := StrToFloat(eP3y.Text);

  Strecken.setP(0,p0x,p0y); Strecken.setP(1,p1x,p1y);
  Strecken.setP(2,p2x,p2y); Strecken.setP(3,p3x,p3y);
  canvas.MoveTo(Round(p0x),Round(p0y));
  canvas.Pen.Color := clRed;
  canvas.LineTo(Round(p1x),Round(p1y));
  canvas.Pen.Color := clGreen;
  canvas.moveTo(Round(p2x),Round(p2y));
  canvas.LineTo(Round(p3x),Round(p3y));
  if Strecken.getIntersect then
    canvas.TextOut(Round(p3x),Round(p3y),'x')
  else
    canvas.TextOut(Round(p3x),Round(p3y),'0');
end;

end.
